#include "network_vol.h"

#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <assert.h>
#include <mercury.h>
#include <mercury_macros.h>

/*
 * Private
 */
int hg_proc_ret_t(hg_proc_t proc, void *data);
int hg_proc_servid_t(hg_proc_t proc, void *data);
int hg_proc_hid_t(hg_proc_t proc, void *data);

MERCURY_GEN_PROC(file_create_in_t, ((hg_string_t)(name)) ((uint32_t)(flags)) 
                 ((hid_t)(fapl_id)) ((hid_t)(fcpl_id)))
MERCURY_GEN_PROC(file_open_in_t, ((hg_string_t)(name)) ((uint32_t)(flags)) 
                 ((hid_t)(fapl_id)))
MERCURY_GEN_PROC(file_open_out_t, ((int64_t)(file_id)) ((hid_t)(fcpl_id)))
/*MERCURY_GEN_PROC(file_flush_in_t, ((iod_handle_t)(coh)) ((int32_t)(scope))
                 ((uint64_t)(axe_id)))

MERCURY_GEN_PROC(attr_create_in_t, ((iod_handle_t)(coh)) ((iod_handle_t)(loc_oh))
                 ((iod_obj_id_t)(loc_id)) ((hg_string_t)(path))
                 ((hg_string_t)(attr_name)) ((hid_t)(acpl_id)) 
                 ((hid_t)(type_id)) ((hid_t)(space_id)) ((uint64_t)(axe_id)))
MERCURY_GEN_PROC(attr_create_out_t, ((iod_handle_t)(iod_oh)) ((iod_obj_id_t)(iod_id)) 
                 ((iod_handle_t)(scratch_oh)) ((iod_obj_id_t)(scratch_id)))
MERCURY_GEN_PROC(attr_open_in_t, ((iod_handle_t)(coh)) ((iod_handle_t)(loc_oh))
                 ((iod_obj_id_t)(loc_id)) ((hg_string_t)(path)) 
                 ((hg_string_t)(attr_name)) ((uint64_t)(axe_id)))
MERCURY_GEN_PROC(attr_open_out_t, ((iod_handle_t)(iod_oh)) ((iod_obj_id_t)(iod_id)) 
                 ((iod_handle_t)(scratch_oh)) ((iod_obj_id_t)(scratch_id))
                 ((hid_t)(acpl_id)) ((hid_t)(type_id)) ((hid_t)(space_id)))
MERCURY_GEN_PROC(attr_op_in_t, ((iod_handle_t)(coh)) ((iod_handle_t)(loc_oh))
                 ((iod_obj_id_t)(loc_id)) ((hg_string_t)(path)) 
                 ((hg_string_t)(attr_name)) ((uint64_t)(axe_id)))
MERCURY_GEN_PROC(attr_io_in_t, ((iod_handle_t)(iod_oh)) ((iod_handle_t)(scratch_oh)) 
                 ((hid_t)(type_id)) ((hg_bulk_t)(bulk_handle)) ((uint64_t)(axe_id)))
MERCURY_GEN_PROC(attr_close_in_t, ((iod_handle_t)(iod_oh)) ((iod_obj_id_t)(iod_id)) 
                 ((iod_handle_t)(scratch_oh)) ((iod_obj_id_t)(scratch_id)) ((uint64_t)(axe_id)))*/

MERCURY_GEN_PROC(group_create_in_t, ((int64_t)(loc_id)) ((hg_string_t)(name))
                 ((hid_t)(gapl_id)) ((hid_t)(gcpl_id)) ((hid_t)(lcpl_id)))
MERCURY_GEN_PROC(group_open_in_t, ((int64_t)(loc_id)) ((hg_string_t)(name))
                 ((hid_t)(gapl_id)))
MERCURY_GEN_PROC(group_open_out_t, ((int64_t)(grp_id)) ((hid_t)(gcpl_id)))

MERCURY_GEN_PROC(dset_create_in_t, ((int64_t)(loc_id)) ((hg_string_t)(name))
                 ((hid_t)(dapl_id)) ((hid_t)(dcpl_id)) ((hid_t)(lcpl_id))
                 ((hid_t)(type_id)) ((hid_t)(space_id)))
MERCURY_GEN_PROC(dset_open_in_t, ((int64_t)(loc_id)) ((hg_string_t)(name))
                 ((hid_t)(dapl_id)))
MERCURY_GEN_PROC(dset_open_out_t, ((int64_t)(dset_id)) ((hid_t)(dcpl_id))
                 ((hid_t)(type_id)) ((hid_t)(space_id)))
/*MERCURY_GEN_PROC(dset_set_extent_in_t, ((iod_handle_t)(iod_oh)) ((dims_t)(dims)) 
                 ((uint64_t)(axe_id)))*/
MERCURY_GEN_PROC(dset_io_in_t, ((int64_t)(dset_id)) ((hid_t)(mem_type_id))
                 ((hid_t)(mem_space_id)) ((hid_t)(file_space_id))
                 ((hid_t)(dxpl_id)) ((hg_bulk_t)(bulk_handle)))

/*MERCURY_GEN_PROC(dtype_commit_in_t, ((iod_handle_t)(coh)) ((iod_handle_t)(loc_oh))
                 ((iod_obj_id_t)(loc_id)) ((hg_string_t)(name))
                 ((hid_t)(tapl_id)) ((hid_t)(tcpl_id)) ((hid_t)(lcpl_id))
                 ((hid_t)(type_id)) ((uint64_t)(axe_id)))
MERCURY_GEN_PROC(dtype_commit_out_t, ((iod_handle_t)(iod_oh)) ((iod_obj_id_t)(iod_id)) 
                 ((iod_handle_t)(scratch_oh)) ((iod_obj_id_t)(scratch_id)))
MERCURY_GEN_PROC(dtype_open_in_t, ((iod_handle_t)(coh)) ((iod_handle_t)(loc_oh))
                 ((iod_obj_id_t)(loc_id)) ((hg_string_t)(name))
                 ((hid_t)(tapl_id)) ((uint64_t)(axe_id)))
MERCURY_GEN_PROC(dtype_open_out_t, ((iod_handle_t)(iod_oh)) ((iod_obj_id_t)(iod_id)) 
                 ((iod_handle_t)(scratch_oh)) ((iod_obj_id_t)(scratch_id))
                 ((hid_t)(tcpl_id)) ((hid_t)(type_id)))
MERCURY_GEN_PROC(dtype_close_in_t, ((iod_handle_t)(iod_oh)) ((iod_obj_id_t)(iod_id)) 
                 ((iod_handle_t)(scratch_oh)) ((iod_obj_id_t)(scratch_id)) ((uint64_t)(axe_id)))*/


#define NVOL_MAX_IDLE_TIME HG_MAX_IDLE_TIME


#define NVOL_DEF_ID -2
#define NVOL_DEF_TYPE -2


#define TRUE 1
#define FALSE 0

#define SUCCEED 0
#define FAIL    -1
#define PRINT_ERROR(MSG)                                                       \
do {                                                                           \
    fprintf(stderr, "ERROR at %s:%d in %s()...\n%s\n", __FILE__, __LINE__, __FUNCTION__,MSG); \
} while(0)
#define ERROR(MSG)                                                             \
do {                                                                           \
    PRINT_ERROR(MSG);                                                          \
    goto error;                                                                \
} while(0)

